import sys
from helpers_hull import *

# global parameters
depth_search = False
img_count = 0

# sweep parameters
step = 50
angle_step = 10
tolerance = 30
overlap_thresh = 60


class Node:
    def __init__(self, _cnt_list, _img_shape, _parent=None):
        # display image with contours
        self.cnt_list = _cnt_list
        self.img_shape = _img_shape
        self.parent = _parent
        self.children = []
        self.check_level()

    def draw_matches(self, img, line, matches):
        global img_count

        img_copy = cv2.bitwise_not(img.copy())
        slope, intercept = line
        x1, y1 = 0, int(intercept)
        x2, y2 = img.shape[1], int(slope * img.shape[1] + intercept)
        cv2.line(img_copy, (x1, y1), (x2, y2), (0, 0, 255), 2)

        for pair in matches:
            cv2.drawContours(img_copy, pair, -1, (0, 0, 255), 2)

        # Display result
        cv2.imshow('Result', img_copy)
        cv2.waitKey(0)
        cv2.destroyAllWindows()
        # Save to disk
        # cv2.imwrite(f'results/out{img_count}.png', img_copy)
        # img_count += 1

    def check_level(self):
        level_checked = False
        while not level_checked:
            line, matches, avg_overlap = get_best_symmetry_line(
                self.cnt_list, self.img_shape, step, angle_step, tolerance, overlap_thresh)
            if line:
                # line found
                self.line = line
                self.matches = matches
                self.avg_overlap = avg_overlap
                self.draw_matches(img, line, matches)

                if depth_search:
                    # extending children
                    left = []
                    right = []
                    for m in matches:
                        # if contour is not reflected on to itself
                        if not np.array_equal(m[0], m[1]):
                            left.append(m[1])
                            right.append(m[0])
                    if right:
                        # one or more matches found
                        self.children.append(Node(right, self.img_shape, self))

                # breath search - removing matched contours

                # Get a flat list of matched contours
                matched_cnts = [cnt for pair in matches for cnt in pair]

                # Replace the old list with a new list that excludes the matched contours
                self.cnt_list = [cnt for cnt in self.cnt_list if not any(
                    np.array_equal(cnt, matched_cnt) for matched_cnt in matched_cnts)]

                if not self.cnt_list:
                    # no more contours left
                    level_checked = True

            else:
                # nothing found
                print('no symmetry found')
                level_checked = True


def get_cnt_list(img):
    # convert the image to grayscale format
    img_temp = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    # gaussian blur
    img_temp = cv2.GaussianBlur(img_temp, (5, 5), cv2.BORDER_DEFAULT)
    # erosion
    kernel = np.ones((3, 3), np.uint8)
    # img_eroded = cv2.erode(img_blurred, kernel, 1)
    # apply binary thresholding
    ret, thresh = cv2.threshold(img_temp, 100, 255, cv2.THRESH_BINARY)

    # detect contours
    cnt_list, hierarchy = cv2.findContours(
        image=thresh, mode=cv2.RETR_LIST, method=cv2.CHAIN_APPROX_SIMPLE)

    return list(cnt_list)


if __name__ == '__main__':

    # Load the image
    img = cv2.imread(sys.argv[1])
    cnts = get_cnt_list(img)
    print(f'{len(cnts)} contours found')

    # display image with contours
    blank = np.full(img.shape, 255, dtype=np.uint8)
    cv2.drawContours(blank, cnts, -1, (0, 0, 0), 2)
    cv2.imshow('Contours', blank)
    cv2.waitKey(0)
    cv2.destroyAllWindows()
    # store to disk
    # cv2.imwrite('cnts.png', blank)

    # Initialize the root node
    root = Node(cnts, img.shape)
